// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancyxmlsockiostream.h 91 2008-01-21 14:38:55Z wglas $
**
** Copyright (C) 2005 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Helper classes to serialize XML/HTML data into an iostream.
** 
*****************************************************************************/

#ifndef FANCY_XMLSOCK_IOSTREAM_H
#define FANCY_XMLSOCK_IOSTREAM_H

#include <iostream>
#include <fancyxmlsockencoding.h>

namespace Fancy {
  namespace Sock {

    FANCYXMLSOCK_API std::ostream &xmlQuote(std::ostream &o, const char *s);
    FANCYXMLSOCK_API std::ostream &xmlQuote(std::ostream &o, const std::string &s);
    FANCYXMLSOCK_API std::ostream &xmlAttrQuote(std::ostream &o, const std::string &s);

    class FANCYXMLSOCK_API XmlQuote
    {
      std::string str;
    public:
      XmlQuote(const char *s) : str(s) {}
      XmlQuote(const std::string &s) : str(s) {}
    
      const std::string &getStr() const { return str; }
    };

    // This allows us to write os << XmlQuote(str)
    inline std::ostream &operator<< (std::ostream &o, const XmlQuote &q)
    {
      return xmlQuote(o,q.getStr());
    }

    class FANCYXMLSOCK_API XmlAttrQuote
    {
      std::string str;
    public:
      XmlAttrQuote(const std::string &s) : str(s) {}
    
      const std::string &getStr() const { return str; }
    };

    // This allows us to write os << XmlQuote(str)
    inline std::ostream &operator<< (std::ostream &o, const XmlAttrQuote &q)
    {
      return xmlAttrQuote(o,q.getStr());
    }

    class FANCYXMLSOCK_API Attribute
    {
    public:
      Attribute(const std::string &k, const std::string &v)
        : key(k),value(v)
      {}

      Attribute(const unsigned char *k, const std::string &v)
        : key(k ? (const char*)k : ""),value(v)
      {}

      Attribute(const std::string &k, const unsigned char *v)
        : key(k),value(v ? (const char*)v : "")
      {}

      Attribute(const unsigned char *k, const unsigned char *v)
        : key(k ? (const char*)k : ""),value(v ? (const char*)v : "")
      {}

      Attribute(const XmlString &k, const std::string &v)
        : key(k.xml_str() ? (const char*)k.xml_str() : ""),value(v)
      {}

      Attribute(const std::string &k, const XmlString &v)
        : key(k),value(v.xml_str() ? (const char*)v.xml_str() : "")
      {}

      Attribute(const XmlString &k, const XmlString &v)
        : key(k.xml_str() ? (const char*)k.xml_str() : ""),value(v.xml_str() ? (const char*)v.xml_str() : "")
      {}

      const std::string &getKey() const { return key; }
      const std::string &getValue() const { return value; }
    private:
      std::string key;
      std::string value;
    };

    class FANCYXMLSOCK_API AttributeList
    {
    public:
      AttributeList();
      ~AttributeList();

      AttributeList &operator << (const Attribute &attr);

      std::string str() const;
    private:
      std::stringstream *s;
    };

    class FANCYXMLSOCK_API Tag
    {
    public:
      Tag(std::ostream &o, const std::string &t, bool empty = false);
      Tag(std::ostream &o, const std::string &t, const AttributeList &l, bool empty = false);
      Tag(std::ostream &o, const std::string &t, const Attribute &a, bool empty = false);

      Tag(const Tag &o, const std::string &t, bool empty = false);
      Tag(const Tag &o, const std::string &t, const AttributeList &l, bool empty = false);
      Tag(const Tag &o, const std::string &t, const Attribute &a, bool empty = false);

      ~Tag();

      // This is used in order to circumvent a parse error with gcc-3.3
      // in Expressions like: Tag(os,"abc") << "Hello, World!";
      Tag &me() { return *this; }
      std::ostream &getStream() const { return *os; }
      const std::string &getTag() const { return tag; }
    private:
      Tag(const Tag &);
      Tag &operator= (const Tag &);

      std::string tag;
      std::ostream *os;
    };

    inline Tag &operator<< (Tag &t, const char *s)
    { xmlQuote(t.getStream(),s); return t; }

    inline Tag &operator<< (Tag &t, const std::string &s)
    { xmlQuote(t.getStream(),s); return t; }

    inline Tag &operator<< (Tag &t, char i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, short i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, unsigned short i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, int i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, unsigned i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, long i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, unsigned long i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, double i)
    { t.getStream() << i; return t; }
  }
}

#endif
