// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancyxmlsockloops.h 91 2008-01-21 14:38:55Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Highlevel socket loop classes with xml packet capabilities.
** 
*****************************************************************************/

#ifndef FANCY_XMLSOCK_LOOPS_H
#define FANCY_XMLSOCK_LOOPS_H

#include <iosfwd>
#include <libxml/tree.h>
#include <fancytimeval.h>
#include <fancythread.h>
#include <fancyxmlsock.h>

namespace Fancy {
  namespace Sock {

    // fwd decls.
    class Address;
    class Device;
    class Protocol;

    class FANCYXMLSOCK_API BaseLoop
    {
    public:
      BaseLoop (Protocol *proto);

      virtual ~BaseLoop();

      void connect (const Address &address);

      void connectionLoop();

      void writeXmlDoc (xmlDocPtr doc);

      void setMaxTimeout(int msec);

      bool isClosed() const;

      virtual void statusInfo(std::ostream &os) const;

    protected:
      virtual void loopStarted();
      virtual int getTimeout(const struct timeval &now);
      virtual int xmlDocReceived (const struct timeval &now, xmlDocPtr doc);
      virtual void peerTimeoutReceived (const struct timeval &now);
      virtual void myTimeoutReceived (const struct timeval &now);

      Device *getDevice() const;
      Protocol *getProtocol() { return protocol; }

    private:
      Protocol *protocol;
      mutable Mutex doc_mutex;
      int max_timeout;
      mutable Mutex close_mutex;
      int closing;
      time_t connect_time;
      int packets_read;
      int packets_written;
    };

    class FANCYXMLSOCK_API HeartbeatLoop : public BaseLoop
    {
    public:
      HeartbeatLoop (Protocol *proto);

      void setHeartbeatInterval(int msec);
      void setHeartbeatTolerance(int msec);

    protected:
      void loopStarted();
      int getTimeout(const struct timeval &now);
      int xmlDocReceived (const struct timeval &now, xmlDocPtr doc);
      void peerTimeoutReceived (const struct timeval &now);
      void myTimeoutReceived (const struct timeval &now);

      void touch();

    private:
      struct timeval heartbeat_interval;
      struct timeval heartbeat_tolerance;
      struct timeval next_heatbeat_recv_tv;
      struct timeval next_heatbeat_send_tv;
    };

  }
}

#endif // ! FANCY_XMLSOCK_LOOPS_H
